Parallel Execution Engine — Rayon-based parallel query execution
This module provides a parallel execution engine that leverages Rayon’s thread pool to execute queries in parallel, improving performance for large workloads.
Structs
- DbConfig
- 데이터베이스 전체 설정
- Parallel
Execution Engine - Parallel execution engine using Rayon thread pool
- Parallelism
Config - 병렬 처리 설정 — CPU 사용량 상한 및 임계값 제어
Enums
- Dirty
Buffer Mode - WOS
dirty버퍼에 사용할 자료구조 선택. - Parallelization
Policy - Parallelization policy for the execution engine