pub struct ParallelExecutionEngine { /* private fields */ }Expand description
Parallel execution engine using Rayon thread pool
Implementations
Sourceimpl ParallelExecutionEngine
impl ParallelExecutionEngine
Sourcepub fn new(policy: ParallelizationPolicy) -> DbxResult<Self>
pub fn new(policy: ParallelizationPolicy) -> DbxResult<Self>
Create a new parallel execution engine with the specified policy
Sourcepub fn new_auto() -> DbxResult<Self>
pub fn new_auto() -> DbxResult<Self>
Create a new parallel execution engine with automatic thread count
Sourcepub fn new_fixed(num_threads: usize) -> DbxResult<Self>
pub fn new_fixed(num_threads: usize) -> DbxResult<Self>
Create a new parallel execution engine with a fixed number of threads
Sourcepub fn policy(&self) -> ParallelizationPolicy
pub fn policy(&self) -> ParallelizationPolicy
Get the current parallelization policy
Sourcepub fn thread_count(&self) -> usize
pub fn thread_count(&self) -> usize
Get the number of threads in the thread pool
Sourcepub fn thread_pool(&self) -> &ThreadPool
pub fn thread_pool(&self) -> &ThreadPool
Get a reference to the thread pool
Sourcepub fn auto_tune(&self, workload_size: usize) -> usize
pub fn auto_tune(&self, workload_size: usize) -> usize
Auto-tune the thread count based on workload size
Returns the recommended number of parallel tasks for the given workload size
Sourcepub fn auto_tune_weighted(
&self,
workload_size: usize,
avg_complexity: f64,
) -> usize
pub fn auto_tune_weighted( &self, workload_size: usize, avg_complexity: f64, ) -> usize
Auto-tune with complexity weight factor
Higher complexity = fewer items per thread needed to justify parallelism
Sourcepub fn estimate_query_complexity(sql: &str) -> f64
pub fn estimate_query_complexity(sql: &str) -> f64
Estimate SQL query complexity based on heuristics
Returns a complexity score (1.0 = simple SELECT, higher = more complex)
Sourcepub fn should_parallelize(&self, workload_size: usize) -> bool
pub fn should_parallelize(&self, workload_size: usize) -> bool
Check if parallelization is beneficial for the given workload size