pub struct TimestampOracle { /* private fields */ }Expand description
A source of monotonically increasing timestamps for MVCC.
This oracle ensures that every transaction gets a unique logical timestamp for determining visibility and commit ordering.
read_ts: The point in time a transaction reads from (snapshot).commit_ts: The point in time a transaction writes its changes.
Implementations