Schema Builder - Fluent API for building Arrow Schemas
This module provides a type-safe, fluent API for building Arrow schemas without manually constructing Field objects.
Structs
- Schema
Builder - Schema Builder for constructing Arrow schemas using a fluent API